package cn.newfeifan.mall.module.trade.service.price;

import cn.newfeifan.mall.module.product.api.sku.ProductSkuApi;
import cn.newfeifan.mall.module.product.api.sku.dto.ProductSkuRespDTO;
import cn.newfeifan.mall.module.product.api.spu.ProductSpuApi;
import cn.newfeifan.mall.module.product.api.spu.dto.ProductSpuRespDTO;
import cn.newfeifan.mall.module.product.dal.dataobject.sku.ProductSkuDO;
import cn.newfeifan.mall.module.product.dal.dataobject.spu.ProductSpuDO;
import cn.newfeifan.mall.module.product.dal.mysql.sku.ProductSkuMapper;
import cn.newfeifan.mall.module.product.dal.mysql.spu.ProductSpuMapper;
import cn.newfeifan.mall.module.trade.service.price.bo.TradePriceCalculateReqBO;
import cn.newfeifan.mall.module.trade.service.price.bo.TradePriceCalculateRespBO;
import cn.newfeifan.mall.module.trade.service.price.calculator.TradeDeliveryPriceCalculator;
import cn.newfeifan.mall.module.trade.service.price.calculator.TradePriceCalculator;
import cn.newfeifan.mall.module.trade.service.price.calculator.TradePriceCalculatorHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.framework.common.util.collection.CollectionUtils.convertMap;
import static cn.newfeifan.mall.framework.common.util.collection.CollectionUtils.convertSet;
import static cn.newfeifan.mall.module.product.enums.ErrorCodeConstants.SKU_NOT_EXISTS;
import static cn.newfeifan.mall.module.product.enums.ErrorCodeConstants.SKU_STOCK_NOT_ENOUGH;
import static cn.newfeifan.mall.module.trade.enums.ErrorCodeConstants.PRICE_CALCULATE_PAY_PRICE_ILLEGAL;

/**
 * 价格计算 Service 实现类
 *
 * @author 非繁源码
 */
@Service
@Validated
@Slf4j
public class TradePriceServiceImpl implements TradePriceService {

    @Resource
    private ProductSkuApi productSkuApi;
    @Resource
    private ProductSpuApi productSpuApi;

    @Resource
    private List<TradePriceCalculator> priceCalculators;

    @Resource
    private ProductSkuMapper productSkuMapper;
    @Resource
    private ProductSpuMapper productSpuMapper;

    /**
     * add by Ben
     * 计算订单含积分的各项商品价格
     *
     * @return 订单价格
     */
    @Override
    public TradePriceCalculateRespBO calculatePriceWithPoints(TradePriceCalculateReqBO calculateReqBO) {
        // 1.1 获得商品 SKU 数组
        List<ProductSkuRespDTO> skuList = checkSkuList(calculateReqBO);
        // 1.2 获得商品 SPU 数组
        List<ProductSpuRespDTO> spuList = checkSpuList(skuList);

        // 2.1 计算价格
        TradePriceCalculateRespBO calculateRespBO = TradePriceCalculatorHelper
                .buildCalculateResp(calculateReqBO, spuList, skuList);
        return calculateRespBO;
    }


    @Override
    public TradePriceCalculateRespBO calculatePrice(TradePriceCalculateReqBO calculateReqBO) {
        // 1.1 获得商品 SKU 数组
        List<ProductSkuRespDTO> skuList = checkSkuList(calculateReqBO);
        // 1.2 获得商品 SPU 数组
        List<ProductSpuRespDTO> spuList = checkSpuList(skuList);

        // 2.1 计算价格
        TradePriceCalculateRespBO calculateRespBO = TradePriceCalculatorHelper
                .buildCalculateResp(calculateReqBO, spuList, skuList);

//        原来下面会循环计算vip价格等，因为用不上，所以改为只计算运算 add by Ben
//        priceCalculators.forEach(calculator -> calculator.calculate(calculateReqBO, calculateRespBO));
        //计算运费
        TradeDeliveryPriceCalculator tradeDeliveryPriceCalculator = null;
        for (TradePriceCalculator tpc : priceCalculators) {
            //把计算运费对象取出来，这个对象不能new，因为自己new会缺少地址信息
            if (tpc.getClass().getName().equals("cn.newfeifan.mall.module.trade.service.price.calculator.TradeDeliveryPriceCalculator")) {
                tradeDeliveryPriceCalculator = (TradeDeliveryPriceCalculator) tpc;
                break;
            }
        }
        calculateRespBO.setShopId(calculateReqBO.getShopId());
        tradeDeliveryPriceCalculator.calculate(calculateReqBO, calculateRespBO);

        // 2.2  如果最终支付金额小于等于 0，则抛出业务异常
        ProductSkuDO sku = productSkuMapper.selectById(calculateReqBO.getItems().get(0).getSkuId());
        ProductSpuDO spu = productSpuMapper.selectById(sku.getSpuId());
        if (!spu.getHighPrecision()) {
            if (calculateRespBO.getPrice().getPayPrice() <= 0) {
                log.error("[calculatePrice][价格计算不正确，请求 calculateReqDTO({})，结果 priceCalculate({})]",
                        calculateReqBO, calculateRespBO);
                throw exception(PRICE_CALCULATE_PAY_PRICE_ILLEGAL);
            }
        }
        return calculateRespBO;
    }

    private List<ProductSkuRespDTO> checkSkuList(TradePriceCalculateReqBO reqBO) {
        // 获得商品 SKU 数组
        Map<Long, Integer> skuIdCountMap = convertMap(reqBO.getItems(),
                TradePriceCalculateReqBO.Item::getSkuId, TradePriceCalculateReqBO.Item::getCount);
        List<ProductSkuRespDTO> skus = productSkuApi.getSkuList(skuIdCountMap.keySet());

        // 校验商品 SKU
        skus.forEach(sku -> {
            Integer count = skuIdCountMap.get(sku.getId());
            if (count == null) {
                throw exception(SKU_NOT_EXISTS);
            }
            if (count > sku.getStock()) {
                throw exception(SKU_STOCK_NOT_ENOUGH);
            }
        });
        return skus;
    }

    private List<ProductSpuRespDTO> checkSpuList(List<ProductSkuRespDTO> skuList) {
        // 获得商品 SPU 数组
        return productSpuApi.validateSpuList(convertSet(skuList, ProductSkuRespDTO::getSpuId));
    }

}
