package cn.newfeifan.mall.module.member.service.address;

import cn.hutool.core.collection.CollUtil;
import cn.newfeifan.mall.module.member.controller.app.address.vo.AppAddressCreateReqVO;
import cn.newfeifan.mall.module.member.controller.app.address.vo.AppAddressUpdateReqVO;
import cn.newfeifan.mall.module.member.convert.address.AddressConvert;
import cn.newfeifan.mall.module.member.dal.dataobject.address.MemberAddressDO;
import cn.newfeifan.mall.module.member.dal.mysql.address.MemberAddressMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.member.enums.ErrorCodeConstants.ADDRESS_NOT_EXISTS;

/**
 * 用户收件地址 Service 实现类
 *
 * @author 非繁源码
 */
@Service
@Validated
@Slf4j
public class AddressServiceImpl implements AddressService {

    @Resource
    private MemberAddressMapper memberAddressMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createAddress(Long userId, AppAddressCreateReqVO createReqVO) {
        // 如果添加的是默认收件地址，则将原默认地址修改为非默认
        if (Boolean.TRUE.equals(createReqVO.getDefaultStatus())) {
            List<MemberAddressDO> addresses = memberAddressMapper.selectListByUserIdAndDefaulted(userId, true);
            addresses.forEach(address -> memberAddressMapper.updateById(new MemberAddressDO().setId(address.getId()).setDefaultStatus(false)));
        }

        // 插入
        MemberAddressDO address = AddressConvert.INSTANCE.convert(createReqVO);
        address.setUserId(userId);
        memberAddressMapper.insert(address);
        // 返回
        return address.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateAddress(Long userId, AppAddressUpdateReqVO updateReqVO) {
        // 校验存在,校验是否能够操作
        validAddressExists(userId, updateReqVO.getId(),updateReqVO.getAddressType());

        // 如果修改的是默认收件地址，则将原默认地址修改为非默认
        if (Boolean.TRUE.equals(updateReqVO.getDefaultStatus())) {
            List<MemberAddressDO> addresses = memberAddressMapper.selectListByUserIdAndDefaulted(userId, true);
            addresses.stream().filter(u -> !u.getId().equals(updateReqVO.getId())) // 排除自己
                    .forEach(address -> memberAddressMapper.updateById(new MemberAddressDO().setId(address.getId()).setDefaultStatus(false)));
        }

        // 更新
        MemberAddressDO updateObj = AddressConvert.INSTANCE.convert(updateReqVO);
        memberAddressMapper.updateById(updateObj);
    }

    @Override
    public void deleteAddress(Long userId, Long id,Integer addressType) {
        // 校验存在,校验是否能够操作
        validAddressExists(userId, id,addressType);
        // 删除
        memberAddressMapper.deleteById(id);
    }

    private void validAddressExists(Long userId, Long id,Integer addressType) {
        MemberAddressDO addressDO = getAddress(userId, id,addressType);
        if (addressDO == null) {
            throw exception(ADDRESS_NOT_EXISTS);
        }
    }

    @Override
    public MemberAddressDO getAddress(Long userId, Long id,Integer addressType) {
        return memberAddressMapper.selectByIdAndUserId(id, userId,addressType);
    }

    @Override
    public List<MemberAddressDO> getAddressList(Long userId,Integer addressType) {
        return memberAddressMapper.selectList(MemberAddressDO::getUserId,userId,MemberAddressDO::getAddressType,addressType);
    }

    @Override
    public MemberAddressDO getDefaultUserAddress(Long userId,Integer addressType) {
        List<MemberAddressDO> addresses;
        log.info("addressType:{}",addressType);
        addresses = memberAddressMapper.selectListByUserIdAndDefaulted(userId, true,addressType);
        return CollUtil.getFirst(addresses);
    }

}
