package cn.newfeifan.mall.module.member.service.user;

import cn.newfeifan.mall.framework.common.enums.TerminalEnum;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.validation.Mobile;
import cn.newfeifan.mall.module.member.controller.admin.user.vo.MemberUserPageReqVO;
import cn.newfeifan.mall.module.member.controller.admin.user.vo.MemberUserUpdateReqVO;
import cn.newfeifan.mall.module.member.controller.app.user.vo.*;
import cn.newfeifan.mall.module.member.dal.dataobject.user.MemberUserDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 会员用户 Service 接口
 *
 * @author 非繁源码
 */
public interface MemberUserService {

    /**
     * 通过手机查询用户
     *
     * @param mobile 手机
     * @return 用户对象
     */
    MemberUserDO getUserByMobile(String mobile);

    /**
     * 基于用户昵称，模糊匹配用户列表
     *
     * @param nickname 用户昵称，模糊匹配
     * @return 用户信息的列表
     */
    List<MemberUserDO> getUserListByNickname(String nickname);

    /**
     * 基于手机号创建用户。
     * 如果用户已经存在，则直接进行返回
     *
     * @param mobile     手机号
     * @param username   用户名
     * @param registerIp 注册 IP
     * @param terminal   终端 {@link TerminalEnum}
     * @return 用户对象
     */
    MemberUserDO createUserIfAbsent(@Mobile String mobile, String username, String registerIp, Integer terminal);

    /**
     * 创建用户
     * 目的：三方登录时，如果未绑定用户时，自动创建对应用户
     *
     * @param mobile     手机号
     * @param nickname   昵称
     * @param avtar      头像
     * @param registerIp 注册 IP
     * @param terminal   终端 {@link TerminalEnum}
     * @param linkId     链接编号
     * @param password   密码
     * @return 用户对象
     */
    MemberUserDO createUser(String mobile, String nickname, String avtar, String registerIp, Integer terminal, Long linkId, String username, String password);

    /**
     * 更新用户的最后登陆信息
     *
     * @param id      用户编号
     * @param loginIp 登陆 IP
     */
    void updateUserLogin(Long id, String loginIp);

    /**
     * 通过用户 ID 查询用户
     *
     * @param id 用户ID
     * @return 用户对象信息
     */
    MemberUserDO getUser(Long id);

    /**
     * 通过用户 ID 查询用户们
     *
     * @param ids 用户 ID
     * @return 用户对象信息数组
     */
    List<MemberUserDO> getUserList(Collection<Long> ids);

    /**
     * 【会员】修改基本信息
     *
     * @param userId 用户编号
     * @param reqVO  基本信息
     */
    void updateUser(Long userId, AppMemberUserUpdateReqVO reqVO);

    /**
     * 【会员】修改手机，基于手机验证码
     *
     * @param userId 用户编号
     * @param reqVO  请求信息
     */
    void updateUserMobile(Long userId, AppMemberUserUpdateMobileReqVO reqVO);

    /**
     * 【会员】修改手机，基于微信小程序的授权码
     *
     * @param userId 用户编号
     * @param reqVO  请求信息
     */
    void updateUserMobileByWeixin(Long userId, AppMemberUserUpdateMobileByWeixinReqVO reqVO);

    /**
     * 【会员】修改密码
     *
     * @param userId 用户编号
     * @param reqVO  请求信息
     */
    void updateUserPassword(Long userId, AppMemberUserUpdatePasswordReqVO reqVO);

    /**
     * 【会员】忘记密码
     *
     * @param reqVO 请求信息
     */
    void resetUserPassword(AppMemberUserResetPasswordReqVO reqVO);

    /**
     * 判断密码是否匹配
     *
     * @param rawPassword     未加密的密码
     * @param encodedPassword 加密后的密码
     * @return 是否匹配
     */
    boolean isPasswordMatch(String rawPassword, String encodedPassword);

    /**
     * 【管理员】更新会员用户
     *
     * @param updateReqVO 更新信息
     */
    void updateUser(@Valid MemberUserUpdateReqVO updateReqVO);

    /**
     * 【管理员】获得会员用户分页
     *
     * @param pageReqVO 分页查询
     * @return 会员用户分页
     */
    PageResult<MemberUserDO> getUserPage(MemberUserPageReqVO pageReqVO);

    /**
     * 更新用户的等级和经验
     *
     * @param id         用户编号
     * @param levelId    用户等级
     * @param experience 用户经验
     */
    void updateUserLevel(Long id, Long levelId, Integer experience);

    /**
     * 获得指定用户分组下的用户数量
     *
     * @param groupId 用户分组编号
     * @return 用户数量
     */
    Long getUserCountByGroupId(Long groupId);

    /**
     * 获得指定用户等级下的用户数量
     *
     * @param levelId 用户等级编号
     * @return 用户数量
     */
    Long getUserCountByLevelId(Long levelId);

    /**
     * 获得指定会员标签下的用户数量
     *
     * @param tagId 用户标签编号
     * @return 用户数量
     */
    Long getUserCountByTagId(Long tagId);

    /**
     * 更新用户的积分
     *
     * @param userId 用户编号
     * @param point  积分数量
     * @return 更新结果
     */
    boolean updateUserPoint(Long userId, Integer point);

    /**
     * 根据用户名查询用户
     * @param username 用户名
     * @return 用户数量
     */
    Long selectCountByUsername(String username);

    /**
     * 根据用户名查询用户
     * @param username 用户名
     * @return 用户
     */
    MemberUserDO getUserByUsername(String username);

    /**
     * 根据手机号查询用户的数量
     * @param mobile 手机号
     * @return 数量
     */
    Long getUserCountByMobile(String mobile);

    MemberUserDO selectByUsername(String username);

    /**
     * 根据系统用户ID查询用户
     * @param systemUserId 系统用户ID
     * @return 用户
     */
    List<MemberUserDO> getUserListBySystemUserId(Long systemUserId);

    /**
     * 根据手机号查询用户
     * @param mobile 手机号
     * @param userId 用户编号
     * @return 用户
     */
    MemberUserDO getUserByMobile(String mobile, Long userId);

    /**
     * 更新用户支付宝账号
     * @param loginUserId 用户编号
     * @param reqVO 请求信息
     */
    void updateUserByAlipayAccount(Long loginUserId, AppMemberUserUpdateAlipayAccountReqVO reqVO);

    /**
     * 更新用户支付宝账号
     * @param loginUserId 用户编号
     * @param reqVO 请求信息
     */
    void updateUserByBankAccount(Long loginUserId, AppMemberUserUpdateBankAccountReqVO reqVO);

    List<MemberUserDO> selectListByMobile(String mobile);

    /**
     * 根据用户名和手机号查询用户
     * @param username 用户名
     * @param mobile 电话
     * @return 用户
     */
    MemberUserDO getUserByUserNameWithMobile(String username,String mobile);

    /**
     * 添加用户注册ID
     * @param registerId 注册ID
     */
    void addRegisterId(String registerId);

    /**
     * 获取用户是否关注公众号
     * @param userId 用户ID
     * @return 是否关注
     */
    Boolean getUserIsSubscribe(Long userId);

    /**
     * 通过系统用户或者会员用户来获取对应的rsgId
     * @param systemUserIds 系统用户
     * @param userId 会员用固话
     * @return rsgId列表
     */
    List<String> getUserRsgIds(Long systemUserIds, Long userId);

    /**
     * 根据用户ID获取用户名
     * @param userId 用户ID
     * @return 用户名
     */
    String getUsernameByUserId(Long userId);

    /**
     * 注销用户
     * @param loginUserId 用户id
     */
    void deleteUserById(Long loginUserId);
}
